/** fMSX: portable MSX emulator ******************************/
/**                                                         **/
/**            HelpX.h (previously named  Help.h)           **/
/**                                                         **/
/** This file contains help information printed out by the  **/
/** main() routine when started with option "-help".        **/
/** Including some extra options for the improved           **/ 
/** X11-drivers by Arnold Metselaar                         **/
/**                                                         **/
/** Copyright (C) Marat Fayzullin  1994,1995                **/
/**               Arnold Metselaar 1996-1999                **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify me, if you make any    **/
/**     changes to this file.                               **/
/*************************************************************/

char *HelpText[] =
{
  "Usage: fmsx [-option1 [-option2...]] [filename1] [filename2] [filename3]",
  "[filename1] = name of file to load as cartridge A",
  "[filename2] = name of file to load as cartridge B",
  "[filename3] = name of file to load as cartridge C",
#ifdef ZLIB
  "  This program will transparently uncompress singular GZIPped",
  "  and PKZIPped files.",
#endif
  "[-option]   =",
  /* generic options: */
  "  -verbose <level>    - Select debugging messages [1]",
  "                        0 - Silent       1 - Startup messages",
  "                        2 - V9938 ops    4 - Disk/Tape",
  "                        8 - Memory      16 - Illegal Z80 ops", 
  "  -hperiod <period>   - Minimal number of CPU cycles per HBlank [228]",
  "  -vperiod <period>   - Minimal number of CPU cycles per VBlank [59736]",
  "  -uperiod <period>   - Number of VBlanks per screen update [2]",
  "  -pal/-ntsc          - Set PAL/NTSC HBlank/VBlank periods [NTSC]",
  "  -help               - Print this help page",
  "  -home <dirname>     - Set directory with system ROM files [off]",
  "  -printer <filename> - Redirect printer output to file [stdout]",
  "  -serial <filename>  - Redirect serial I/O to a file [stdin/stdout]",
  "  -diska <filename>   - Set disk image used for drive A: [DRIVEA.DSK]",
  "                        (multiple -diska options accepted)",
  "  -diska \"[\" [fn1 [fn2...]] \"]\"",
  "                      - Set a list of disk images to use for drive A:",
  "                        (equivalent to multiple -diska options)",
  "  -diskb <filename>   - Set disk image used for drive B: [DRIVEB.DSK]",
  "                        (multiple -diskb options accepted)",
  "  -diskb \"[\" [fn1 [fn2...]] \"]\"",
  "                      - Set a list of disk images to use for drive B:",
  "                        (equivalent to multiple -diskb options)",
  "  -tape <filename>    - Set tape image file [off]",
  "  -font <filename>    - Set fixed font for text modes [DEFAULT.FNT]",
  "  -logsnd <filename>  - Write soundtrack to a file [LOG.MID]",
  "  -auto/-noauto       - Use autofire on SPACE [off]",
  "  -ram <pages>        - Number of 16kB RAM pages [4/8/8]",
  "  -vram <pages>       - Number of 16kB VRAM pages [2/8/8]",
  "  -rom <type>         - Select MegaROM mapper types [6,6]",
  "                        (two -rom options accepted)",
  "                        0 - generic 8kB   1 - generic 16kB",
  "                        2 - Konami5 8kB   3 - Konami4 8kB",
  "                        4 - ASCII 8kB     5 - ASCII 16kB",
  "                        6 - GameMaster2   7 - FMPAC",
  "                        >7 - Try guessing mapper type",
  "  -msx1/-msx2/-msx2+  - Select MSX model [-msx2]",
  "  -joy <type>         - Select joystick types [0,0]",
  "                        (two -joy options accepted)",
  "                        0 - No joystick",
  "                        1 - Normal joystick",
  "                        2 - Mouse in joystick mode",
  "                        3 - Mouse in real mode",

#ifdef DEBUG
  "  -trap <address>     - Trap execution when PC reaches address [FFFFh]",
  "                        (when keyword 'now' is used in place of the",
  "                        <address>, execution will trap immediately)",
#endif /* DEBUG */

#ifdef SOUND
#ifdef MSDOS
  "  -sound [<quality>]  - Sound emulation quality [1]",
#else
  "  -sound [<quality>]  - Sound emulation quality [22000]",
#endif
  "                        0 - Off                1 - Adlib (MSDOS)",
  "                        Values >8191 are treated as wave synthesis",
  "                        frequencies.",
  "  -nosound            - Same as '-sound 0'",
#endif /* SOUND */

#ifdef UNIX
#ifdef MITSHM
  "  -shm/-noshm         - Use MIT SHM extensions for X [-shm]",
#endif
  "  -saver/-nosaver     - Save CPU when inactive [-saver]",
  "  -sync <frequency>   - Sync screen updates to <frequency> [0]",
  "  -nosync             - Same as '-sync 0'",
  "  -scale <factor>     - Scale window by <factor> [1]",
#endif /* UNIX */

#ifdef MSDOS
  "  -vsync/-novsync     - Sync screen updates [-novsync]",
  "  -static/-nostatic   - Use static color allocation [-nostatic]",
  "  -240/-200           - Use non-standard 256x240 mode [-200]",
#endif /* MSDOS */
 /* Options specific to screen drivers: */
#ifdef MACH_HELP_H
#include MACH_HELP_H
#else
  "",
  "  -keybuf/-nokeybuf   - Pass keystrokes one at a time [-nokeybuf]",
  "  -allspr             - Don't limit number of visible sprites per line",
  "  -normspr            - Limit sprites per line like a real MSX [default]",
  "  -limspr <n>         - Show at most n sprites on a line",
  "  -bw                 - Black and white",
  "  -ncol <n>           - Allocate at most n colors",
  "  -install/-noinstall - Install private colormap [-noinstall]",
  "  -display <dpy>      - Use display dpy",
  "  -depth <n>          - Set depth for visual",
  "  -visual <type>      - Set visual-class for visual",
  "  -visualid <id>      - Set visualid id for visual",
  "  -screen <n>         - Set screen for visual",
  "  -synchronise        - Synchronise connection to X-server(for debugging)",
#endif /* MACH_HELP_H */

  /* help about keyboard handler/key bindings: */
#ifdef KEYB_HELP_H
#include KEYB_HELP_H
#else 
  "\nKeyboard bindings:",

#ifdef UNIX
  "  [KEYPAD_Ins]    - Joystick A button",
  "  [KEYPAD_Del]    - Joystick B button",
  "  [R_CONTROL]     - CAPS LOCK",
  "  [L_ALT]         - GRAPH",
  "  [R_ALT]         - COUNTRY",
  "  [F6]            - Turn spacebar autofire on/off",
  "  [F7]            - Load emulation from .STA file",
  "  [CONTROL]+[F7]  - Save emulation state to .STA file",
  "  [F8]            - Change disk in drive A:",
  "  [CONTROL]+[F8]  - Change disk in drive B:",
  "  [F9]            - Fast-forward emulation",
  "  [CONTROL]+[F9]  - Turn fixed font on/off",
#ifdef SOUND
  "  [F10]           - Turn sound on/off",
#endif
  "  [CONTROL]+[F10] - Turn soundtrack logging on/off",
#ifdef DEBUG
  "  [F11]           - Go into the built-in debugger",
#endif
  "  [F12]           - Quit emulation",
#endif /* UNIX */

#ifdef MSDOS
  "  [PGUP]          - STOP/BREAK",
  "  [PGDOWN]        - SELECT",
  "  [F6]            - Turn spacebar autofire on/off",
  "  [F7]            - Change disk in drive A:",
  "  [F8]            - Change disk in drive B:",
  "  [F9]            - Turn fixed font on/off",
  "  [F10]           - Turn soundtrack logging on/off",
#ifdef GIFLIB
  "  [F11]           - Make a screen snapshot (SNAPxxxx.GIF)",
#endif
  "  [F12]           - Quit emulation",
#endif /* MSDOS */
#endif /* KEYB_HELP_H */
  0
};

char * HelpOnCommands [] =
{
  "You can type can type one or more commands on a line.",
  "Type an empty line to make the emulation continue.",
  "Possible commands:",
  "  verbose <level>     - Select debugging messages [1]",
  "                        0 - Silent       1 - Startup messages",
  "                        2 - V9938 ops    4 - Disk",
  "                        8 - Memory      16 - Illegal Z80 ops", 
  "  ?                   - Print this help page",
  "  restart             - Also continues emulation",
  "  uperiod <period>    - Set number of VBlanks per screen update",
  "  printer <filename>  - Redirect printer output to file",
  "  serial <filename>   - Redirect serial I/O to a file",
  "  diska <filename>    - Set disk image used for drive A:",
  "  diskb <filename>    - Set disk image used for drive B:",
  "  tape <filename>     - Set tape image file",
  "  rom <type>[,<type>] - Select MegaROM mapper types",
  "                        0 - generic 8kB   1 - generic 16kB",
  "                        2 - Konami5 8kB   3 - Konami4 8kB",
  "                        4 - ASCII 8kB     5 - ASCII 16kB",
  "                        6 - GameMaster2",
#if 0 /* I need to fix this */
  "  sound/nosound       - turn sound on/off",
#endif
#ifdef UNIX
  "  saver/nosaver       - Save CPU when inactive",
#endif
  /* Commands specific to screen drivers: */
#ifdef MACH_COM_H
#include MACH_COM_H
#else
  "  allspr              - Don't limit number of visible sprites per line",
  "  normspr             - Limit sprites per line like a real MSX",
  "  limspr <n>          - Show at most n sprites on a line",
  "  ncol <n>            - Allocate at most n colors",
#endif
  /* command specific to keyboard handler: */ 
#ifdef KEYB_COM_H
#include KEYB_COM_H
#else
  "  keybuf/nokeybuf     - Pass keystrokes one at a time",
#endif
  0
}; 

