/** fMSX: portable MSX emulator ******************************/
/**                                                         **/
/**               fMSXx.c (previously named fMSX.c)         **/
/**                                                         **/
/** This file contains main() procedure starting            **/
/** the emulation and Doption procedure for handling        **/
/** changes to options. To be used together with the        **/
/** improved X11-drivers written by Arnold Metselaar.       **/
/**                                                         **/
/**                                                         **/
/** Copyright (C) Marat Fayzullin 1994-2000                 **/
/**               Arnold Metselaar 1998-2000                **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify us, if you make any    **/
/**     changes to this file.                               **/
/*************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#ifdef BROKEN_ERRNO
extern const char *const sys_errlist[];
#endif

#ifndef O_BINARY
#define O_BINARY 0
#endif

#define True 1
#define False 0

#include "MSX.h"
#include "HelpX.h"

/* prototypes: */
static void NewStream(char *, FILE **, char *, char *); /* purpose,
							   fp*, name, mode */
void DOption();
/* in AnyX.c: */
int MachineOption (int argc, char *argv[], int *N);
int MachineCommand(int argc, char *argv[], int *N);

extern char *Title;
#ifdef SOUND
extern int   UseSound;
#endif
#ifdef MITSHM
extern int   UseSHM;
#endif
extern int   SaveCPU;
extern int   IFreq;
extern int   KeyBuf; 
extern FILE *PrnStream, *CasStream;
#if 0 /* def UNIX */
extern int ncol;
extern int NSprMax_M, NSprMax_C;
extern int do_install, do_sync, do_mono;
extern char* Dispname;
#endif

static char *Options[]=
{ 
  "verbose","hperiod","vperiod","uperiod","pal","ntsc","help",
  "printer","serial","diska","diskb","tape","font","logsnd","state",
  "ram","vram","rom","auto","noauto","msx1","msx2","msx2+","joy",
  "home","trap","sound","nosound","shm","noshm","saver","nosaver",
  "sync","nosync","scale","vsync","novsync","240","200","static",
  "nostatic",0
};

extern char *Title;      /* Program title                       */
extern int   UseSound;   /* Sound mode (#ifdef SOUND)           */
extern int   UseSHM;     /* Use SHM X extension (#ifdef MITSHM) */
extern int   UseZoom;    /* Zoom factor (#ifdef UNIX)           */
extern int   SaveCPU;    /* Pause when inactive (#ifdef UNIX)   */
extern int   SyncFreq;   /* Sync screen updates (#ifdef UNIX)   */

/** Zero-terminated arrays of disk names for each drive ******/
extern char *Disks[2][MAXDISKS+1];
extern int CurDisk[2]; /* indices to current disks */

/* for the extra options: */
extern int NSprMax_M,NSprMax_C;

/** main() ***************************************************/
/** This is a main() function used in Unix and MSDOS ports. **/
/** It parses command line arguments, sets emulation        **/
/** parameters, and passes control to the emulation itself. **/
/*************************************************************/
int 
main(int argc,char *argv[])
{
  int N,J,CartCount,TypeCount,JoyCount, DiskCount[2];

#ifdef DEBUG
  CPU.Trap=0xFFFF;CPU.Trace=0;
#endif

#ifdef MSDOS
  Verbose=0;
#else
  Verbose=1;
#endif

  /* Clear everything */
  CartCount=TypeCount=JoyCount=0;
  DiskCount[0]=DiskCount[1]=0;
  NSprMax_M=4;
  NSprMax_C=8;

  /* Default disk images */
  Disks[0][1]=Disks[1][1]=0;
  Disks[0][0]=DSKName[0];
  Disks[1][0]=DSKName[1];

  for(N=1;N<argc;N++)
    if(*argv[N]!='-')
    {
      if(CartCount<MAXCARTS) ROMName[CartCount++]=argv[N];
      else printf("%s: Excessive filename '%s'\n",argv[0],argv[N]);
    }
    else
    {    
  	/* Is the option specific to the drivers ? */  
      if (MachineOption(argc,argv,&N))
	continue;

      for(J=0;Options[J];J++)
        if(!strcmp(argv[N]+1,Options[J])) break;

      switch(J)
      {
        case 0:  N++;
                 if(N<argc) Verbose=atoi(argv[N]);
                 else printf("%s: No verbose level supplied\n",argv[0]);
                 break;
        case 1:  N++;
                 if(N>=argc)
                   printf("%s: No HBlank period supplied\n",argv[0]);
                 else
                 {
                   J=atoi(argv[N]);
                   if((J>=30)&&(J<=3000)) HPeriod=J;
                 }
                 break;
        case 2:  N++;
                 if(N>=argc)
                   printf("%s: No VBlank period supplied\n",argv[0]);
                 else
                 {
                   J=atoi(argv[N]);
                   if((J>=1000)&&(J<=100000)) VPeriod=J;
                   if((J>=256)&&(J<=999)) VPeriod=HPeriod*J;
                 }
                 break;
        case 3:  N++;
                 if(N>=argc)
                   printf("%s: No screen update period supplied\n",argv[0]);
                 else
                 {
                   J=atoi(argv[N]);
                   if((J>=1)&&(J<=20)) UPeriod=J; 
                 }
                 break;
        case 4:  VPeriod=VPERIOD_PAL/6;
                 HPeriod=HPERIOD/6;
                 break;
        case 5:  VPeriod=VPERIOD_NTSC/6;
                 HPeriod=HPERIOD/6;
                 break;
	case 6:  printf
  		 ("%s by Marat Fayzullin    (C)FMS 1994-2004\n",Title);
	         puts("Improved X-drivers by Arnold Metselaar "); 
                 for(J=0;HelpText[J];J++) puts(HelpText[J]);
                 return(0);
        case 7:  N++;
                 if(N<argc) PrnName=argv[N];
                 else printf("%s: No file for printer redirection\n",argv[0]);
                 break;
        case 8:  N++;
                 if(N<argc) ComName=argv[N];
                 else printf("%s: No file for serial redirection\n",argv[0]);
                 break;
        case 9:
        case 10:
	  N++;
	  if(N>=argc)
	    printf("%s: No file for drive %c\n",argv[0],'A'+J-9);
	  else
	    if (strcmp(argv[N],"[")==0) 
	      {
		while ((++N<argc)&&(strcmp(argv[N],"]")!=0))
		  {
		    if(DiskCount[J-9]>=MAXDISKS)
  		    {
  		      printf("%s: Too many disks for drive %c\n",
  			     argv[0],'A'+J-9);
  		      break;
  		    }
		    else
		      Disks[J-9][DiskCount[J-9]++]=argv[N];
		  }
		if (N==argc)
		  printf("%s: missing ]\n",argv[0]);
	      }
	    else
	      if(DiskCount[J-9]>=MAXDISKS)
		printf("%s: Too many disks for drive %c\n",argv[0],'A'+J-9);
	      else
		Disks[J-9][DiskCount[J-9]++]=argv[N];
	  break;
        case 11: N++;  
  	         if(N<argc) CasName=argv[N];
		 else printf("%s: No file for the tape\n",argv[0]);
		 break;
        case 12: N++;
                 if(N<argc) FontName=argv[N];  
                 else printf("%s: No font name supplied\n",argv[0]);
                 break;
        case 13: N++;
                 if(N<argc) SndName=argv[N];
                 else printf("%s: No file for soundtrack logging\n",argv[0]);
                 break;
        case 14: N++;
	         if(N<argc) StateName=argv[N];
                 else printf("%s: No file to save emulation state\n",argv[0]);
                 break;
        case 15: N++;
                if(N>=argc)
                   printf("%s: No number of RAM pages supplied\n",argv[0]);
                 else
                   RAMPages=atoi(argv[N]);
                 break;
        case 16: N++;
                 if(N>=argc)
                   printf("%s: No number of VRAM pages supplied\n",argv[0]);
                 else
                   VRAMPages=atoi(argv[N]);
                 break;
        case 17: N++;  
                 if(N>=argc)
                   printf("%s: No ROM mapper type supplied\n",argv[0]);
                 else
                   if(TypeCount<MAXCARTS)
                     ROMType[TypeCount++]=atoi(argv[N]);
                   else
                     printf("%s: Excessive -rom option\n",argv[0]);
                 break;
        case 18: AutoFire=1;break;
        case 19: AutoFire=0;break;
        case 20: MSXVersion=0;break;
        case 21: MSXVersion=1;break;
        case 22: MSXVersion=2;break;
        case 23: N++;
                 if(N>=argc)
                   printf("%s: No joystick type supplied\n",argv[0]);
                 else
                   switch(JoyCount++)
                   {
                     case 0: JoyTypeA=atoi(argv[N]);break;
                     case 1: JoyTypeB=atoi(argv[N]);break;
                     default: printf("%s: Excessive -joy option\n",argv[0]);
                   }
                 break;
        case 24: N++;
                 if(N>=argc)
                   printf("%s: No home directory name supplied\n",argv[0]);
                 else
                   ProgDir=argv[N];
                 break;
#ifdef DEBUG
        case 25: N++;
                 if(N>=argc)
                   printf("%s: No trap address supplied\n",argv[0]);
                 else
                   if(!strcmp(argv[N],"now")) CPU.Trace=1;
                   else sscanf(argv[N],"%hX",&(CPU.Trap));
                 break;
#endif
#ifdef SOUND
        case 26: N++;
                 if(N>=argc) { UseSound=1;N--; }
                 else if(sscanf(argv[N],"%d",&UseSound)!=1)
                      { UseSound=1;N--; }
                 break;
        case 27: UseSound=0;break;
#endif
#ifdef UNIX
#ifdef MITSHM
        case 28: UseSHM=1;break;
        case 29: UseSHM=0;break;
#endif
        case 30: SaveCPU=1;break;
        case 31: SaveCPU=0;break;
        case 32: N++;
                 if(N<argc) SyncFreq=atoi(argv[N]);
                 else printf("%s: No sync frequency supplied\n",argv[0]);
                 break;
        case 33: SyncFreq=0;break;
        case 34: N++;
  	         if(N<argc) UseZoom=atoi(argv[N]);
		 else printf("%s: No scaling factor supplied\n",argv[0]);
		 break;
#endif
#ifdef MSDOS
        case 35: SyncScreen=1;break;
        case 36: SyncScreen=0;break;
        case 37: FullScreen=1;break;
        case 38: FullScreen=0;break;
        case 39: UseStatic=1;break;
        case 40: UseStatic=0;break;
#endif
        default: printf("%s: Wrong option '%s'\n",argv[0],argv[N]);
  	}
    }
  
  /* Terminate disk lists and set initial disk names */
  if(DiskCount[0]) { Disks[0][DiskCount[0]]=0;DSKName[0]=Disks[0][0]; }
  if(DiskCount[1]) { Disks[1][DiskCount[1]]=0;DSKName[1]=Disks[1][0]; }

  /* Start fMSX! */
  if(!InitMachine()) return(1);
  StartMSX();
  TrashMSX();
  TrashMachine();
  return(0);
}

char *Commands[]=
{
  "verbose","?","quit","restart","uperiod","printer","rom",
  "diska","diskb","tape", "saver","nosaver","sound","nosound",
/* the following options are only here because fmsxif by shigeki itoh
   may send them, their arguments will be ignored */
  "size", "rensha", "volume", 
  0
};

void 
DOption()
{
  char linbuf[256], *p;
  char *tokp[128];
  int Count, N, J, a, b;
  char c;
  
  tokp[0]="fmsx: command processor";
  printf("\n--entering fMSX command mode--\nType '?' for help\n");
  do {
      printf("fMSX>"); fflush(NULL);
/* read line and split into tokens: */

      p=fgets(linbuf, 256, stdin);
      if (p==NULL) {
	  if (feof(stdin))
	      printf("fmsx: end of file on stdin\n");
	  else
	      perror("fmsx: can't read stdin");	  
	  break;
      }
      Count=1;
      while (*p) {
	  while (isspace(*p)) 
	      p++;
	  if (*p=='\0') break;

	  if((*p=='\'')||(*p=='"')) {
	      c=*p++;
	      tokp[Count++]=p;
	      while((*p)&&(*p!=c)) 
		  p++;
	  }
	  else {
	      tokp[Count++]=p;
	      while ((*p) && !isspace(*p)) 
		  p++;
	  }
	  *p++='\0';
      } 
/* show input: 
      for (N=0 ; N<Count ; N++)
          printf("%d:`%s'\n",N,tokp[N]); 
*/
/* proces input: */
      for (N=1 ; N<Count ; N++) {
	if (MachineCommand(Count,tokp,&N))
	  continue;
	for (J=0 ; Commands[J] && strcmp(Commands[J],tokp[N]) ; J++)
	  ;
	switch (J) {
	case  0:
	  N++;
	  if (N<Count) Verbose=atoi(tokp[N]);
	  else printf("%s: No verbose level supplied\n",tokp[0]);
	  break;
	case  1:
	    for (a=0 ; HelpOnCommands[a] ; a++)
	      puts( HelpOnCommands[a] );
	break;
	case  2:
	  ExitNow=1;  /* stopcondition for fmsx */
	  Count=0;/* stopcondition for do ... while  */
	  break;
	case  3:
	  Count=0;/* stopcondition for do ... while  */
	  break;
  	case  4:
	  N++;
	  if(N>=Count)
	    printf("%s: No screen update period supplied\n",tokp[0]);
	  else {
	    UPeriod=atoi(tokp[N]);
	    if((UPeriod<1)||(UPeriod>20)) UPeriod=2;
	  }
	  break;
	case  5: 
	  N++;
	  if(N<Count)
	    { 
	      PrnName=tokp[N];
	      NewStream("for printer redirectioning",&PrnStream,
			PrnName,"wb");
	    }
	  else printf("%s: No file for printer redirection\n",tokp[0]);
	  break;
	case  6:
	  N++;
	  switch ((N>=Count)?0:sscanf(tokp[N],"%d,%d,%d",&a,&b,&c))
	    {
	    case 0:
	      printf("%s: No ROM mapper type supplied\n",tokp[0]);
	      break;
	    case 1: 
	      b=a;
	    case 2:
	      c=b;
	    case 3: 
	      if ((a>=0)&&(a<MAXMAPPERS)) ROMType[0]= (char)a;
	      if ((b>=0)&&(b<MAXMAPPERS)) ROMType[1]= (char)b;
	      if ((c>=0)&&(c<MAXMAPPERS)) ROMType[2]= (char)b;
	      printf("Romtypes set to: %d,%d,%d\n",
		     ROMType[0],ROMType[1],ROMType[2]);
	    }
	  break;
	case  7:
	case  8:
	  N++; a=J-7;
	  if (N<Count) {
	    for ( b=0 ; Disks[a][b] && (b<MAXDISKS) ; b++)
	      if (strcmp(tokp[N],Disks[a][b])==0) break;
	    
	    if (Disks[a][b])
	      ChangeDisk(a,Disks[a][CurDisk[a]=b]);
	    else
	      {
		/* don't handle this gracefully for now */
		if (b>=MAXDISKS)
		  printf("%s: Too many disks for drive %c\n",tokp[0],'A'+a);
		else
		  {
		    CurDisk[a]=b;
		    Disks[a][b++]=
		      strcpy(malloc(strlen(tokp[N]+1)),tokp[N]);
		    ChangeDisk(a,tokp[N]);
		    printf("Inserting %s into drive %c\n",tokp[N],'A'+a);
	    
		    Disks[a][b]=0;
		  }
	      }
	  }
	  else printf("%s: No file for drive %c\n",tokp[0], 'A'+a);
	  break;
	case  9: 
	  N++;  
	  if (N<Count) { 
	    CasName=tokp[N];
	    NewStream("for tape",&CasStream,CasName,"r+b");
	  }
	  else printf("%s: No file for the tape\n",tokp[0]);
	  break;
	case 10: 
	  if ((N+1<Count) && (sscanf(tokp[N+1],"%d",&a)==1)) {
	    SaveCPU=a;
	    N++;
	  } else {
	    SaveCPU=1;
	  }
	  break;
	case 11: SaveCPU=0; break;
#ifdef SOUND
	  /* this needs to be fixed
	     case 12: UseSound=1; break;
	     case 13: UseSound=0; break; */
#endif
	case 12: 
	case 13:
	case 14:
	  printf("%s: '%s %s' ignored\n", tokp[0], tokp[N], tokp[N+1]);
	    N++;
	    break;
        default: 
	  printf("%s: Wrong command '%s'\n",tokp[0],tokp[N]);
	}
      }
  } while(Count>1);
  printf("\n--leaving fMSX commmand mode--\n");
}

static void 
NewStream(char * purpose, FILE ** stream, char * name, char * mode)
{
   if ((*stream!=stdout) && (*stream!=stdin) && *stream)
       fclose(*stream);
   
   printf("Opening %s %s...",name,purpose);
   *stream=fopen(name,mode);
   printf((*stream?"OK\n":"FAILED\n"));
}
